
import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

public class Echo extends HandlerBase
{

    static private Writer out;

//**********************************************//

    public void startDocument()
	throws SAXException
    {
	nl();
	nl();
	emit ( "START DOCUMENT" );
	emit ( "<?xml version='1.0' encoding='UTF-8'?>" );
	nl();

    } // END void startDocument () 

//**********************************************//

    public void endDocument()
	throws SAXException
    {
	nl();
	emit ( "END DOCUMENT" );

	try
	{
	    nl();
	    out.flush();

	} // end try
	catch ( IOException e )
	{
	    throw new SAXException ( "I/O error", e );

	} // end catch

    } // END void endDocument()

//**********************************************//

    public void startElement ( String name, AttributeList attrs )
	throws SAXException
    {
	nl();

	emit ( "<" + name );

	if ( attrs != null )
	{
	    for ( int i = 0; i < attrs.getLength(); i++ )
	    {
		nl();

		emit ( "   ATTR: " );
		emit ( attrs.getName(i) );
		emit ( "\t\"" );
		emit ( attrs.getValue(i) );
		emit ( "\"" );

	    } // end for

	} // end if

	if ( attrs.getLength() > 0 )
	{
	    nl();
	    emit ( "</" + name + ">" );

	} // end if

    } // END void startElement ( String name, AttributeList attrs )

//**********************************************//

    public void endElement ( String name )
	throws SAXException
    {
	nl();
	emit ( "END ELM: " );
	emit ( "</" + name + ">" );

    } // END void endElement ( String name )

//**********************************************//

    public void characters ( char buf [], int offset, int len )
	throws SAXException
    {
	nl();
	emit ( "CHARS:  " );

	String s = new String ( buf, offset, len );

	if ( ! s.trim().equals ( "" ) )
	{
	    emit ( s );

	} // end if

    } // END void characters ( char buf [], int offset, int len )

//**********************************************//

    private void emit ( String s )
	throws SAXException
    {
	try
	{
	    out.write ( s );
	    out.flush();

	} // end try
	catch ( IOException e )
	{
	    throw new SAXException ( "I/O error", e );

	} // end catch

    } // END void emit ( String s )

//**********************************************//

    private void nl()
	throws SAXException
    {
	String lineEnd = System.getProperty ( "line.separator" );

	try
	{
	    out.write ( lineEnd );

	} // end try
	catch ( IOException e )
	{
	    throw new SAXException ( "I/O error", e );

	} // end catch

    } // END void nl()

//**********************************************//

    public static void main ( String argv[] )
    {
	if ( argv.length != 1 )
	{
	    System.err.println ( "Usage: cmd filename" );
	    System.exit ( 1 );

	} // end if argv not 1 long

	// Use the default (non-validating) parser
	SAXParserFactory factory = SAXParserFactory.newInstance();

	try
	{
	    // Set up output stream
	    out = new OutputStreamWriter ( System.out, "UTF8" );

	    // Parse the input
	    SAXParser saxParser = factory.newSAXParser();
	    saxParser.parse ( new File ( argv [0] ),
			      new Echo() );

	} // end try
	catch ( Throwable t )
	{
	    t.printStackTrace();

	} // end catch

	System.exit ( 0 );

    } // END static void main ( String argv[] )

//**********************************************//

} // END class Echo extends HandlerBase
